local AceAddon = LibStub("AceAddon-3.0")
local AceDB = LibStub("AceDB-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local KairosUI = AceAddon:NewAddon("KairosUI")

-- Default settings
KairosUI.defaults = {
	global = {
		moveBuffs = true,
		hideBags = true,
		moveTooltip = true,
		movePortraits = true,
		moveFocus = false,
		additionalActionBar = true,
		hideArenaFrames = true,
		hideDruidShapeshift = false,
		hideRaidFramesInArena = false,
		hidePaladinPower = false,
		hideCastBar = false,
		moveVehicleSeatIndicator = false,
	}
}

-- Options table
KairosUI.optionsTable = {
	type = "group",
	args = {
		description = {
			order = 1,
			type = "description",
			fontSize = "medium",
			name = "UI Modifications & Fixes by Kairos (v" .. GetAddOnMetadata("KairosUI", "Version") .. ")",
		},
		break1 = {
			order = 2,
			type = "header",
			name = "",
		},
		reloadButton = {
			order = 3,
			type = "execute",
			name = "Reload for changes to take effect",
			func = ReloadUI,
			width = 10,
		},
		headerBuffs = {
			order = 4,
			type = "header",
			name = "Buffs",
		},
		moveBuffs = {
			order = 5,
			name = "Move Buffs",
			type = "toggle",
			get = function(info) return KairosUI.db.global.moveBuffs end,
			set = function(info, val) KairosUI.db.global.moveBuffs = val end,
		},
		headerBags = {
			order = 6,
			type = "header",
			name = "Bags",
		},
		hideBags = {
			order = 7,
			name = "Hide Bags",
			type = "toggle",
			get = function(info) return KairosUI.db.global.hideBags end,
			set = function(info, val) KairosUI.db.global.hideBags = val end,
		},
		headerTooltip = {
			order = 8,
			type = "header",
			name = "Tooltip",
		},
		moveTooltip = {
			order = 9,
			name = "Move Tooltip",
			type = "toggle",
			get = function(info) return KairosUI.db.global.moveTooltip end,
			set = function(info, val) KairosUI.db.global.moveTooltip = val end,
		},
		headerPortraits = {
			order = 10,
			type = "header",
			name = "Portraits",
		},
		movePortraits = {
			order = 11,
			name = "Move Portraits",
			desc = "Move Player and Target frames",
			type = "toggle",
			get = function(info) return KairosUI.db.global.movePortraits end,
			set = function(info, val) KairosUI.db.global.movePortraits = val end,
		},
		moveFocus = {
			order = 12,
			name = "Move Focus Frame",
			desc = "Move Focus frame",
			type = "toggle",
			get = function(info) return KairosUI.db.global.moveFocus end,
			set = function(info, val) KairosUI.db.global.moveFocus = val end,
		},
		hidePaladinPower = {
			order = 13,
			name = "Hide Holy Power",
			desc = "Hides default Blizzard Holy Power frame",
			type = "toggle",
			get = function(info) return KairosUI.db.global.hidePaladinPower end,
			set = function(info, val) KairosUI.db.global.hidePaladinPower = val end,
		},
		headerActionBars = {
			order = 14,
			type = "header",
			name = "Action Bars",
		},
		additionalActionBar = {
			order = 15,
			name = "Extra Action Bars",
			desc = "Extra Action Bars at the bottom right (REQUIRES XarMods)",
			type = "toggle",
			get = function(info) return KairosUI.db.global.additionalActionBar end,
			set = function(info, val) KairosUI.db.global.additionalActionBar = val end,
		},
		headerArenaFrames = {
			order = 16,
			type = "header",
			name = "Arena",
		},
		hideArenaFrames = {
			order = 17,
			name = "Hide Arena Frames",
			desc = "Hides Blizzard Arena Frames. Does not affect BG flag carriers",
			type = "toggle",
			get = function(info) return KairosUI.db.global.hideArenaFrames end,
			set = function(info, val) KairosUI.db.global.hideArenaFrames = val end,
		},
		hideRaidFramesInArena = {
			order = 18,
			name = "Hide Raid Frames",
			desc = "Hide Blizzard Raid Frames when in an Arena",
			type = "toggle",
			get = function(info) return KairosUI.db.global.hideRaidFramesInArena end,
			set = function(info, val) KairosUI.db.global.hideRaidFramesInArena = val end,
		},
		headerMisc = {
			order = 19,
			type = "header",
			name = "Miscellaneous",
		},
		hideDruidShapeshift = {
			order = 20,
			name = "Hide Shapeshift Bar",
			type = "toggle",
			get = function(info) return KairosUI.db.global.hideDruidShapeshift end,
			set = function(info, val) KairosUI.db.global.hideDruidShapeshift = val end,
		},
		hideCastBar = {
			order = 21,
			name = "Hide Cast Bar",
			type = "toggle",
			get = function(info) return KairosUI.db.global.hideCastBar end,
			set = function(info, val) KairosUI.db.global.hideCastBar = val end,
		},
		moveVehicleSeatIndicator  = {
			order = 22,
			name = "Move Mount Indicator",
			type = "toggle",
			get = function(info) return KairosUI.db.global.moveVehicleSeatIndicator end,
			set = function(info, val) KairosUI.db.global.moveVehicleSeatIndicator = val end,
		},
	}
}

local eventHandler = CreateFrame("Frame", nil, UIParent)
eventHandler:SetScript("OnEvent", function(self, event, ...) return self[event](self, ...) end)
eventHandler:RegisterEvent("PLAYER_LOGIN")
eventHandler:RegisterEvent("PLAYER_ENTERING_WORLD")

-- PLAYER_LOGIN event
function eventHandler:PLAYER_LOGIN(arg1)

	KairosUI.db = AceDB:New("KairosUIDB", KairosUI.defaults, true)
	
	AceConfig:RegisterOptionsTable("KairosUI", KairosUI.optionsTable)
	AceConfigDialog:AddToBlizOptions("KairosUI")
	
	-- Hide Bags
	if KairosUI.db.global.hideBags then
		CharacterBag0Slot:Hide()
		CharacterBag1Slot:Hide()
		CharacterBag2Slot:Hide()
		CharacterBag3Slot:Hide()
		MicroButtonAndBagsBar:Hide()
	end
	
	-- Hide Druid Shapeshift Bar
	if KairosUI.db.global.hideDruidShapeshift then
		localizedClass, englishClass, classIndex = UnitClass("player")
		if englishClass == "DRUID" then
			StanceBarFrame:Hide()
			RegisterStateDriver(StanceBarFrame, "visibility", "hide")
		end
	end
	
	-- Hide Paladin Power Bar
	if KairosUI.db.global.hidePaladinPower then
		--message("fittytish")
		PaladinPowerBarFrame:Hide()
		RegisterStateDriver(PaladinPowerBarFrame, "visibility", "hide")
	end

	-- Move VehicleSeatIndicator
	if KairosUI.db.global.moveVehicleSeatIndicator then
		local origsetpoint = getmetatable(VehicleSeatIndicator).__index.SetPoint
		local function move(self)
		   self:ClearAllPoints()
		   origsetpoint(self, "TOPRIGHT", MinimapCluster, "BOTTOMRIGHT", -181.99, 176.52)
		end
		hooksecurefunc(VehicleSeatIndicator, "SetPoint", move)
		move(VehicleSeatIndicator)
	end
	
	-- Hide Cast Bar
	if KairosUI.db.global.hideCastBar then
		CastingBarFrame:Hide()
		CastingBarFrame:UnregisterAllEvents() 
	end
	
	-- Move Buff Frame
	if KairosUI.db.global.moveBuffs then

		-- Allow buff frame to be moved
		BuffFrame:SetMovable(true)
		BuffFrame:SetUserPlaced(true)
		BuffFrame:SetDontSavePosition(true)
		BuffFrame:SetClampedToScreen(true)

		-- Set buff frame position at startup
		BuffFrame:ClearAllPoints()
		BuffFrame:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -375, -175)
		BuffFrame:SetScale(1.0)
		
		-- Set buff frame position when the game resets it
		hooksecurefunc("UIParent_UpdateTopFramePositions", function()
			BuffFrame:SetMovable(true)
			BuffFrame:ClearAllPoints()
			BuffFrame:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -375, -175)
		end)
	
		-- Compatibility with Leatrix/SUI
		if rBFS_BuffDragFrame then
		
			-- Move Buff Frame
			rBFS_BuffDragFrame:ClearAllPoints()
			rBFS_BuffDragFrame:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -375, -175)
		
			-- Move Debuff Frame
			rBFS_DebuffDragFrame:ClearAllPoints()
			rBFS_DebuffDragFrame:SetPoint("TOP", rBFS_BuffDragFrame, "BOTTOM", 0, -72)
			
		end
		
	end
	
	-- Move Focus Frame
	if KairosUI.db.global.moveFocus then
		FocusFrame:ClearAllPoints()
		FocusFrame:SetPoint("CENTER", UIParent, "CENTER", -338.59, 48.27)
		FocusFrame:SetScale(1.0)
	end
	
	-- Move Tooltip
	if KairosUI.db.global.moveTooltip then
		hooksecurefunc("GameTooltip_SetDefaultAnchor", function(tooltip, parent)
			tooltip:ClearAllPoints()
			tooltip:SetPoint("BOTTOMRIGHT", MultiBarBottomRightButton12, "TOPRIGHT", 0, 16)
		end)
	end

	-- Move Portraits
	if KairosUI.db.global.movePortraits then
		-- Move PlayerFrame
		PlayerFrame:ClearAllPoints()
		PlayerFrame:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -1250, -155)
		PlayerFrame:SetUserPlaced(true)
		
		-- Move TargetFrame
		TargetFrame:ClearAllPoints()
		TargetFrame:SetPoint("LEFT", PlayerFrame, "RIGHT", -8, 0)
	end

	-- Additional Action Bars
	if KairosUI.db.global.additionalActionBar then
		if IsAddOnLoaded("XarMods") then

			-- Kairos right bars
			
			-- attach left bar button 5 to CharacterMicroButton
			MultiBarLeftButton5:ClearAllPoints()
			MultiBarLeftButton5:SetPoint("BOTTOM", CharacterMicroButton, "TOP", 6, 10)
			
			-- rearrange left bar buttons 6 - 11 horizontally
			MultiBarLeftButton6:ClearAllPoints()
			MultiBarLeftButton6:SetPoint("LEFT", MultiBarLeftButton5, "RIGHT", 6, 0)
			MultiBarLeftButton7:ClearAllPoints()
			MultiBarLeftButton7:SetPoint("LEFT", MultiBarLeftButton6, "RIGHT", 6, 0)
			MultiBarLeftButton8:ClearAllPoints()
			MultiBarLeftButton8:SetPoint("LEFT", MultiBarLeftButton7, "RIGHT", 6, 0)
			MultiBarLeftButton9:ClearAllPoints()
			MultiBarLeftButton9:SetPoint("LEFT", MultiBarLeftButton8, "RIGHT", 6, 0)
			MultiBarLeftButton10:ClearAllPoints()
			MultiBarLeftButton10:SetPoint("LEFT", MultiBarLeftButton9, "RIGHT", 6, 0)
			MultiBarLeftButton11:ClearAllPoints()
			MultiBarLeftButton11:SetPoint("LEFT", MultiBarLeftButton10, "RIGHT", 6, 0)
			
			-- attach right bar button 5 to left bar button 5
			MultiBarRightButton5:ClearAllPoints()
			MultiBarRightButton5:SetPoint("BOTTOM", MultiBarLeftButton5, "TOP", 0, 6)
			
			-- rearrange right bar buttons 6 - 11 horizontally
			MultiBarRightButton6:ClearAllPoints()
			MultiBarRightButton6:SetPoint("LEFT", MultiBarRightButton5, "RIGHT", 6, 0)
			MultiBarRightButton7:ClearAllPoints()
			MultiBarRightButton7:SetPoint("LEFT", MultiBarRightButton6, "RIGHT", 6, 0)
			MultiBarRightButton8:ClearAllPoints()
			MultiBarRightButton8:SetPoint("LEFT", MultiBarRightButton7, "RIGHT", 6, 0)
			MultiBarRightButton9:ClearAllPoints()
			MultiBarRightButton9:SetPoint("LEFT", MultiBarRightButton8, "RIGHT", 6, 0)
			MultiBarRightButton10:ClearAllPoints()
			MultiBarRightButton10:SetPoint("LEFT", MultiBarRightButton9, "RIGHT", 6, 0)
			MultiBarRightButton11:ClearAllPoints()
			MultiBarRightButton11:SetPoint("LEFT", MultiBarRightButton10, "RIGHT", 6, 0)
			
		end
	end
	
	-- Slash Command
	_G["SLASH_KairosUI1"] = "/kairos"
	_G["SLASH_KairosUI2"] = "/kairosui"
	SlashCmdList["KairosUI"] = function() LibStub("AceConfigDialog-3.0"):Open("KairosUI") end
	
end

-- PLAYER_ENTERING_WORLD event
function eventHandler:PLAYER_ENTERING_WORLD(arg1)
	local name, instanceType = GetInstanceInfo()
	
	if KairosUI.db.global.additionalActionBar then
		if instanceType == "arena" then
			ArenaEnemyFrame1:Hide()
			ArenaEnemyFrame2:Hide()
			ArenaEnemyFrame3:Hide()
			ArenaEnemyFrame4:Hide()
			ArenaEnemyFrame5:Hide()
			ArenaPrepFrame1:Hide()
			ArenaPrepFrame2:Hide()
			ArenaPrepFrame3:Hide()
			ArenaPrepFrame4:Hide()
			ArenaPrepFrame5:Hide()
		elseif instanceType == "pvp" then
			ArenaEnemyFrame1:Show()
			ArenaEnemyFrame2:Show()
			ArenaEnemyFrame3:Show()
			ArenaEnemyFrame4:Show()
			ArenaEnemyFrame5:Show()
			ArenaPrepFrame1:Show()
			ArenaPrepFrame2:Show()
			ArenaPrepFrame3:Show()
			ArenaPrepFrame4:Show()
			ArenaPrepFrame5:Show()
		end
	end
	
	if KairosUI.db.global.hideRaidFramesInArena then
		if instanceType == "party" then
			CompactRaidFrameContainer:Hide()
		else
			CompactRaidFrameContainer:Show()
		end
	end
end